import { Vec2 } from "./vector";
export class MouseController {
  private last_event_time = Date.now();
  private last_movement: Vec2 | undefined;
  private interval_id: number | undefined;

  constructor(
    private readonly publisher: (new_movement: Vec2) => void | Promise<void>,
    private readonly debounce_ms = 200,
    private readonly l2_norm_threshold = 40,
  ) {}

  public start() {
    if (typeof this.interval_id !== "undefined") {
      return;
    }
    this.interval_id = setInterval(
      () => this.publish_movement(),
      this.debounce_ms,
    );
  }

  public stop() {
    if (this.interval_id === null) {
      return;
    }
    clearInterval(this.interval_id);
    delete this.interval_id;
  }

  public move(x: number, y: number) {
    const new_movement = new Vec2(x, y);
    if (
      typeof this.last_movement !== "undefined" &&
      new_movement.distance_to(this.last_movement) >= this.l2_norm_threshold
    ) {
      this.publish_movement();
    }
    this.last_movement = new_movement;
  }

  private publish_movement() {
    if (
      typeof this.last_movement === "undefined" ||
      Date.now() - this.last_event_time < this.debounce_ms
    ) {
      return;
    }

    this.last_event_time = Date.now();
    this.publisher(this.last_movement.copy());
    delete this.last_movement;
  }
}
